using System;
using System.Runtime.InteropServices;
using System.Threading;
using System.IO;
using System.Net;
using System.Drawing;
using System.Drawing.Imaging;
using Bespoke.Common.Osc;

namespace Server
{
	class Program
	{
		static void Main(string[] args)
        {
            sSendMessages = true;

            if (TransportType == TransportType.Tcp)
            {
                sOscClient = new OscClient(Destination.Address, Destination.Port);
                sOscClient.Connect();
            }

            sMessage = new OscMessage(new IPEndPoint(IPAddress.Loopback, Port), AliveMethod, sOscClient);

            MemoryStream memoryStream = new MemoryStream();
            Resources.Image.Save(memoryStream, ImageFormat.Jpeg);

            sMessage.Append(memoryStream.ToArray());

            Thread serverThread = new Thread(RunWorker);
            serverThread.Start();

            Console.WriteLine("OSC Transmitter: " + TransportType.ToString());
            Console.WriteLine("Press any key to exit.");
            Console.ReadKey();
            sSendMessages = false;
            serverThread.Join();

            if (TransportType == TransportType.Tcp)
            {
                sOscClient.Close();
            }
        }

		static void RunWorker()
		{
			while (sSendMessages)
			{
                switch (TransportType)
                {
                    case TransportType.Udp:
                        sMessage.Send(Destination);
                        break;

                    case TransportType.Tcp:
                        sMessage.Send();
                        break;
                }
				
				Thread.Sleep(1000);
			}
		}

        private static readonly TransportType TransportType = TransportType.Udp;
        private static readonly IPAddress MulticastAddress = IPAddress.Parse("224.25.26.27");
		private static readonly int Port = 5103;
		private static readonly IPEndPoint Destination = new IPEndPoint(MulticastAddress, Port);
        private static readonly string AliveMethod = "/osctest/alive";

		private static volatile bool sSendMessages;
		private static OscMessage sMessage;
        private static OscClient sOscClient;
	}
}
